@@ -240,16 +240,12 @@ bool Regexp::ParseState::PushRegexp(Regexp* re) {
 // Searches the case folding tables and returns the CaseFold* that contains r.
 // If there isn't one, returns the CaseFold* with smallest f->lo bigger than r.
 // If there isn't one, returns NULL.
-CaseFold* LookupCaseFold(Rune r) {
-  CaseFold *f;
-  int n, m;
-
-  f = unicode_casefold;
-  n = num_unicode_casefold;
+CaseFold* LookupCaseFold(CaseFold *f, int n, Rune r) {
+  CaseFold* ef = f + n;
 
   // Binary search for entry containing r.
   while (n > 0) {
-    m = n/2;
+    int m = n/2;
     if (f[m].lo <= r && r <= f[m].hi)
       return &f[m];
     if (r < f[m].lo) {
@@ -261,10 +257,10 @@ CaseFold* LookupCaseFold(Rune r) {
   }
 
   // There is no entry that contains r, but f points
-  // where it would have been.  Unless f points off
+  // where it would have been.  Unless f points at
   // the end of the array, it points at the next entry
   // after r.
-  if (f < unicode_casefold+num_unicode_casefold)
+  if (f < ef)
     return f;
 
   // No entry contains r; no entry contains runes > r.
@@ -272,16 +268,24 @@ CaseFold* LookupCaseFold(Rune r) {
 }
 
 // Returns the result of applying the fold f to the rune r.
-static Rune ApplyFold(CaseFold *f, Rune r) {
+Rune ApplyFold(CaseFold *f, Rune r) {
   switch (f->delta) {
     default:
       return r + f->delta;
 
+    case EvenOddSkip:  // even <-> odd but only applies to every other
+      if ((r - f->lo) % 2)
+        return r;
+      // fall through
     case EvenOdd:  // even <-> odd
       if (r%2 == 0)
         return r + 1;
       return r - 1;
 
+    case OddEvenSkip:  // odd <-> even but only applies to every other
+      if ((r - f->lo) % 2)
+        return r;
+      // fall through
     case OddEven:  // odd <-> even
       if (r%2 == 1)
         return r + 1;
@@ -300,7 +304,7 @@ static Rune ApplyFold(CaseFold *f, Rune r) {
 //
 //   CycleFoldRune('?') = '?'
 Rune CycleFoldRune(Rune r) {
-  CaseFold* f = LookupCaseFold(r);
+  CaseFold* f = LookupCaseFold(unicode_casefold, num_unicode_casefold, r);
   if (f == NULL || r < f->lo)
     return r;
   return ApplyFold(f, r);
@@ -323,7 +327,7 @@ static void AddFoldedRange(CharClassBuilder* cc, Rune lo, Rune hi, int depth) {
     return;
 
   while (lo <= hi) {
-    CaseFold* f = LookupCaseFold(lo);
+    CaseFold* f = LookupCaseFold(unicode_casefold, num_unicode_casefold, lo);
     if (f == NULL)  // lo has no fold, nor does anything above lo
       break;
     if (lo < f->lo) {  // lo has no fold; next rune with a fold is f->lo
@@ -464,7 +468,7 @@ bool Regexp::ParseState::PushRepeatOp(RegexpOp op, const StringPiece& s,
 bool Regexp::ParseState::PushRepetition(int min, int max,
                                         const StringPiece& s,
                                         bool nongreedy) {
-  if ((max != -1 && max < min) || max > 1000) {
+  if ((max != -1 && max < min) || min > 1000 || max > 1000) {
     status_->set_code(kRegexpRepeatSize);
     status_->set_error_arg(s);
     return false;
@@ -1275,14 +1279,15 @@ static bool ParseEscape(StringPiece* s, Rune* rp,
     case '0':
       // consume up to three octal digits; already have one.
       code = c - '0';
-      c = (*s)[0];
-      if (s->size() > 0 && '0' <= c && c <= '7') {
+      if (s->size() > 0 && '0' <= (c = (*s)[0]) && c <= '7') {
         code = code * 8 + c - '0';
         s->remove_prefix(1);  // digit
-        c = (*s)[0];
-        if (s->size() > 0 && '0' <= c && c <= '7') {
-          code = code * 8 + c - '0';
-          s->remove_prefix(1);  // digit
+        if (s->size() > 0) {
+          c = (*s)[0];
+          if ('0' <= c && c <= '7') {
+            code = code * 8 + c - '0';
+            s->remove_prefix(1);  // digit
+          }
         }
       }
       *rp = code;
@@ -1358,7 +1363,7 @@ static bool ParseEscape(StringPiece* s, Rune* rp,
     // in Perl, \b means word-boundary but [\b]
     // means backspace.  We don't support that:
     // if you want a backspace embed a literal
-    // backspace character or use \008.
+    // backspace character or use \x08.
     //
     // case 'b':
     //   *rp = '\b';
@@ -2153,7 +2158,7 @@ Regexp* Regexp::Parse(const StringPiece& s, ParseFlags global_flags,
           }
         }
 
-        if (t[1] == 'p' || t[1] == 'P') {
+        if (t.size() >= 2 && (t[1] == 'p' || t[1] == 'P')) {
           Regexp* re = new Regexp(kRegexpCharClass, ps.flags() & ~FoldCase);
           re->ccb_ = new CharClassBuilder;
           switch (ParseUnicodeGroup(&t, ps.flags(), re->ccb_, status)) {
