@@ -301,7 +301,7 @@ class Regexp {
                            //   it explicitly.
 
     // As close to Perl as we can get.
-    LikePerl     = ClassNL | PerlClasses | PerlB | PerlX |
+    LikePerl     = ClassNL | OneLine | PerlClasses | PerlB | PerlX |
                    UnicodeGroups,
 
     // Internal use only.
@@ -370,6 +370,11 @@ class Regexp {
   // The caller is responsible for deleting the map.
   map<string, int>* NamedCaptures();
 
+  // Returns a map from capturing group indices to capturing group
+  // names or NULL if the regexp contains no named capture groups. The
+  // caller is responsible for deleting the map.
+  map<int, string>* CaptureNames();
+
   // Returns a string representation of the current regexp,
   // using as few parentheses as possible.
   string ToString();
